#define PGLOGGER_C

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <libpq-fe.h>
#include "pglogger.h"

PGconn *conn;

int log2db(log *values) {
    int retry_count = 0;
    PGresult *res = NULL;
    const char sqlstring[] = "INSERT INTO apachelog (clientaddress,rfc1413,username,\"localtime\",httprequest,statuscode,bytessenttoclient,referer,clientsoftware,timespent,canonname) VALUES ($1,$2,$3,$4,$5,$6,$7,$8,$9,$10,$11)";
    char *sqlvalues[11];
    char statuscode[128];
    char bytessenttoclient[128];
    char timespent[128];
    
    sprintf(statuscode,"%d",values->statuscode);
    sprintf(bytessenttoclient,"%ld",values->bytessenttoclient);
    sprintf(timespent,"%ld",values->timespent);

    sqlvalues[0] = values->rclient;
    sqlvalues[1] = values->rlogname;
    sqlvalues[2] = values->ruser;
    sqlvalues[3] = values->time_buffer;
    sqlvalues[4] = values->the_request;
    sqlvalues[5] = statuscode;
    sqlvalues[6] = bytessenttoclient;
    sqlvalues[7] = values->referer;
    sqlvalues[8] = values->clientsoftware;
    sqlvalues[9] = timespent;
    sqlvalues[10] = values->server_hostname;
    
    res = PQexecParams(conn,sqlstring,11,NULL,(const char **)sqlvalues,NULL,NULL,0);
    while((retry_count < 3) && (PQstatus(conn) == CONNECTION_BAD)) {
	PQclear(res);
	PQreset(conn);
	res = PQexecParams(conn,sqlstring,11,NULL,(const char **)sqlvalues,NULL,NULL,0);
	retry_count++;
    }

    if((PQstatus(conn) == CONNECTION_BAD) || (PQresultStatus(res) != PGRES_COMMAND_OK)) {
	PQclear(res);
	return -1;
    }
    PQclear(res);
    return 0;
}

int opendb(const char *connection) {
    conn = PQconnectdb(connection);
    if (PQstatus(conn) != CONNECTION_BAD)
	return 0;
    else
	return -1;
}

void closedb() {
    PQfinish(conn);
}
